@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2016  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2016  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -423,6 +408,18 @@ static const char *lookup_checksum(enum l_checksum_type checksum)
 	case L_CHECKSUM_SHA512:
 		ret = "sha512";
 		break;
+	case L_CHECKSUM_SHA3_224:
+		ret = "sha3-224";
+		break;
+	case L_CHECKSUM_SHA3_256:
+		ret = "sha3-256";
+		break;
+	case L_CHECKSUM_SHA3_384:
+		ret = "sha3-384";
+		break;
+	case L_CHECKSUM_SHA3_512:
+		ret = "sha3-512";
+		break;
 	}
 
 	return ret;
