@@ -26,8 +26,8 @@
 #include <sys/epoll.h>
 #include <sys/uio.h>
 
-#include "lib/bluetooth.h"
-#include "lib/hci.h"
+#include "bluetooth/bluetooth.h"
+#include "bluetooth/hci.h"
 
 #include "src/shared/mainloop.h"
 #include "btdev.h"
