@@ -12,8 +12,12 @@
 #include <config.h>
 #endif
 
+#include <time.h>
+
 #include <ell/ell.h>
 
+#include "src/shared/ad.h"
+
 #include "mesh/mesh-defs.h"
 #include "mesh/crypto.h"
 #include "mesh/net.h"
@@ -95,7 +99,7 @@ struct idle_rx {
 
 static struct l_queue *pb_sessions = NULL;
 
-static const uint8_t filter[1] = { MESH_AD_TYPE_PROVISION };
+static const uint8_t filter[1] = { BT_AD_MESH_PROV };
 
 static void pb_adv_packet(void *user_data, const uint8_t *pkt, uint16_t len);
 
@@ -128,7 +132,7 @@ static void send_adv_segs(struct pb_adv_session *session, const uint8_t *data,
 							uint16_t size)
 {
 	uint16_t init_size;
-	uint8_t buf[PB_ADV_MTU + 6] = { MESH_AD_TYPE_PROVISION };
+	uint8_t buf[PB_ADV_MTU + 6] = { BT_AD_MESH_PROV };
 	uint8_t max_seg;
 	uint8_t consumed;
 	int i;
@@ -166,7 +170,7 @@ static void send_adv_segs(struct pb_adv_session *session, const uint8_t *data,
 	consumed = init_size;
 
 	for (i = 1; i <= max_seg; i++) {
-		uint8_t seg_size; /* Amount of payload data being sent */
+		size_t seg_size; /* Amount of payload data being sent */
 
 		if (size - consumed > PB_ADV_MTU - 1)
 			seg_size = PB_ADV_MTU - 1;
@@ -234,7 +238,7 @@ static void pb_adv_tx(void *user_data, const void *data, uint16_t len)
 
 static void send_open_req(struct pb_adv_session *session)
 {
-	struct pb_open_req open_req = { MESH_AD_TYPE_PROVISION };
+	struct pb_open_req open_req = { BT_AD_MESH_PROV };
 
 	l_put_be32(session->link_id, &open_req.link_id);
 	open_req.trans_num = 0;
@@ -249,7 +253,7 @@ static void send_open_req(struct pb_adv_session *session)
 
 static void send_open_cfm(struct pb_adv_session *session)
 {
-	struct pb_open_cfm open_cfm = { MESH_AD_TYPE_PROVISION };
+	struct pb_open_cfm open_cfm = { BT_AD_MESH_PROV };
 
 	l_put_be32(session->link_id, &open_cfm.link_id);
 	open_cfm.trans_num = 0;
@@ -263,7 +267,7 @@ static void send_open_cfm(struct pb_adv_session *session)
 
 static void send_ack(struct pb_adv_session *session, uint8_t trans_num)
 {
-	struct pb_ack ack = { MESH_AD_TYPE_PROVISION };
+	struct pb_ack ack = { BT_AD_MESH_PROV };
 
 	if (!l_queue_find(pb_sessions, session_match, session))
 		return;
@@ -278,7 +282,7 @@ static void send_ack(struct pb_adv_session *session, uint8_t trans_num)
 
 static void send_close_ind(struct pb_adv_session *session, uint8_t reason)
 {
-	struct pb_close_ind close_ind = { MESH_AD_TYPE_PROVISION };
+	struct pb_close_ind close_ind = { BT_AD_MESH_PROV };
 
 	if (!l_queue_find(pb_sessions, session_match, session))
 		return;
