@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -7,20 +8,6 @@
  *  Copyright (C) 2005-2007  Johan Hedberg <johan.hedberg@nokia.com>
  *
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -85,9 +72,9 @@ const char *class_to_icon(uint32_t class)
 		switch ((class & 0xfc) >> 2) {
 		case 0x01:
 		case 0x02:
-			return "audio-card";	/* Headset */
+			return "audio-headset";
 		case 0x06:
-			return "audio-card";	/* Headphone */
+			return "audio-headphones";
 		case 0x0b: /* VCR */
 		case 0x0c: /* Video Camera */
 		case 0x0d: /* Camcorder */
