@@ -25,8 +25,8 @@
 
 #include <glib.h>
 
-#include "lib/bluetooth.h"
-#include "lib/hci.h"
+#include "bluetooth/bluetooth.h"
+#include "bluetooth/hci.h"
 
 #ifdef HAVE_VALGRIND_MEMCHECK_H
 #include <valgrind/memcheck.h>
@@ -654,6 +654,8 @@ static void test_result(enum test_result result)
 		test->timeout_id = 0;
 	}
 
+	tester_shutdown_io();
+
 	if (test->result == TEST_RESULT_FAILED)
 		result = TEST_RESULT_FAILED;
 
@@ -978,10 +980,14 @@ static bool test_io_recv(struct io *io, void *user_data)
 	if (!iov)
 		return true;
 
-	g_assert_cmpint(len, ==, iov->iov_len);
+	if (test->iovcnt && !iov->iov_base)
+		iov = test_get_iov(test);
 
-	if (memcmp(buf, iov->iov_base, len))
-		tester_monitor('!', 0x0004, 0x0000, iov->iov_base, len);
+	if (memcmp(buf, iov->iov_base, len) || (size_t)len != iov->iov_len)
+		tester_monitor('!', 0x0004, 0x0000, iov->iov_base,
+							iov->iov_len);
+
+	g_assert_cmpint(len, ==, iov->iov_len);
 
 	g_assert(memcmp(buf, iov->iov_base, len) == 0);
 
@@ -1044,6 +1050,12 @@ struct io *tester_setup_io(const struct iovec *iov, int iovcnt)
 	return ios[0];
 }
 
+void tester_shutdown_io(void)
+{
+	io_shutdown(ios[0]);
+	io_shutdown(ios[1]);
+}
+
 void tester_io_send(void)
 {
 	struct test_case *test = tester_get_test();
